from dataclasses import dataclass


# The result of a version bump operation on a particular file; each result
# represents an individual file along with the details of its version change
@dataclass
class FileResult(object):
    file_path: str
    current_version: str
    new_version: str



#!/usr/bin/env python3

from nose2.tools import params

import bumpanything.__main__ as bump
from tests import (
    BumpAnythingTestCase,
    create_mock_file,
    read_mock_file,
    redirect_stdout,
    use_cli_args,
)


class TestBump(BumpAnythingTestCase):

    @params(
        # Bump the major part of the version
        (("major", "foo.py"), "1.2.3", "2.0.0"),
        # Bump the minor part of the version
        (("minor", "foo.py"), "1.2.3", "1.3.0"),
        # Bump the patch part of the version
        (("patch", "foo.py"), "1.2.3", "1.2.4"),
        # Bump the prerelease part of the version
        (("prerelease", "foo.py"), "1.2.3-beta.4", "1.2.3-beta.5"),
        # Set the version to an explicit new version
        (("4.5.6", "foo.py"), "1.2.3-beta.4", "4.5.6"),
        # Set the version to an explicit new version with prerelease part
        (("7.8.9-alpha.1", "foo.py"), "1.2.3-beta.4", "7.8.9-alpha.1"),
        # Set the version to an explicit new version with prerelease part
        (("7.8.9+post.1", "foo.py"), "1.2.3-beta.4", "7.8.9+post.1"),
    )
    @redirect_stdout
    def test_bump_explicit_file(self, out, cli_args, old_version, new_version):
        """should bump version for various cases, given an explicit file path"""
        file_name = cli_args[-1]
        file_contents = f"""
        name = "foo"
        version = {old_version}
        """
        with use_cli_args(*cli_args):
            create_mock_file(
                file_name,
                file_contents,
            )
            bump.main()
            self.assertIn(
                f"{file_name}: {old_version} -> {new_version}", out.getvalue()
            )
            self.assertIn(f"version = {new_version}\n", read_mock_file(file_name))

    @redirect_stdout
    def test_bump_multiple_files(self, out):
        """should bump version for various cases, given an explicit file path"""
        file_name_1 = "package.json"
        file_name_2 = "package-lock.json"
        old_version = "1.2.3"
        new_version = "4.5.6"
        file_contents_1 = f"""{{
            "name": "foo",
            "version": {old_version}
        }}"""
        file_contents_2 = f"""{{
            "lockfileVersion": 1,
            "name": "foo",
            "version": {old_version}
        }}"""
        with use_cli_args(new_version):
            create_mock_file(file_name_1, file_contents_1)
            create_mock_file(file_name_2, file_contents_2)
            bump.main()
            self.assertIn(
                f"{file_name_1}: {old_version} -> {new_version}", out.getvalue()
            )
            self.assertIn(
                f"{file_name_2}: {old_version} -> {new_version}", out.getvalue()
            )
            self.assertIn(f'"version": {new_version}\n', read_mock_file(file_name_1))
            self.assertIn(f'"version": {new_version}\n', read_mock_file(file_name_2))

    @params(
        # Set the version to the same version
        (("1.2.3", "foo.py"), "1.2.3", "1.2.3"),
    )
    @redirect_stdout
    def test_no_change(self, out, cli_args, old_version, new_version):
        """should not modify file if same version is given"""
        file_name = cli_args[-1]
        file_contents = f"""
        name = "foo"
        version = {old_version}
        """
        with use_cli_args(*cli_args):
            create_mock_file(
                file_name,
                file_contents,
            )
            bump.main()
            self.assertIn("No files updated", out.getvalue())
            self.assertEqual(file_contents, read_mock_file(file_name))



#!/usr/bin/env python3

import bumpanything.__main__ as bump
from tests import (
    BumpAnythingTestCase,
    create_mock_file,
    redirect_stderr,
    redirect_stdout,
    use_cli_args,
)


class TestEdge(BumpAnythingTestCase):

    @redirect_stdout
    @redirect_stderr
    def test_nonexistent_file_explicit(self, err, out):
        """should indicate if the specified file does not exist"""
        with use_cli_args("major", "foo.py"):
            bump.main()
            self.assertIn("foo.py: file not found", err.getvalue())
            self.assertNotIn("foo.py: file not found", out.getvalue())

    @redirect_stdout
    def test_nonexistent_file_implicit(self, out):
        """should indicate if no files were specified and none of the default files
        were found"""
        with use_cli_args("major"):
            bump.main()
            self.assertIn("No files updated", out.getvalue())

    @redirect_stdout
    def test_no_version_found(self, out):
        """should indicate when no version field was found in the specified file"""
        with use_cli_args("major", "foo.py"):
            create_mock_file(
                "foo.py",
                """
                There's no version here.. unless you brought it with you!
            """,
            )
            bump.main()
            self.assertIn("No files updated", out.getvalue())

    @redirect_stdout
    @redirect_stderr
    def test_invalid_version_specifier(self, err, out):
        """should throw an error if supplied version specifier is not a valid
        semantic version or increment type"""
        with use_cli_args("1.2.3_1"):
            with self.assertRaises(SystemExit):
                bump.main()
            self.assertIn("invalid version specifier", err.getvalue())



#!/usr/bin/env python3

import os
import os.path
import subprocess


def is_in_git_repository():
    root_dir = os.path.abspath(os.path.sep)
    current_dir_path = os.getcwd()
    while current_dir_path != root_dir:
        if os.path.exists(os.path.join(current_dir_path, ".git")):
            return True
        current_dir_path = os.path.dirname(current_dir_path)
    return False


def run_git_command(subcommand, *args):
    try:
        print(
            subprocess.check_output(
                ["git", subcommand, *args], stderr=subprocess.STDOUT, text=True
            ),
            end="",
        )
        return True
    except subprocess.CalledProcessError:
        return False


def add(file_paths):
    return run_git_command("add", *file_paths)


def commit(message):
    return run_git_command("commit", "-m", message)


def tag(tag_name):
    return run_git_command("tag", tag_name)



#!/usr/bin/env python3

import argparse
import os
import os.path
import re
import sys
from functools import partial

import semver

import bumpanything.git as git
from bumpanything.file_result import FileResult

# The regular expression pattern used to match the version to be incremented
# within any given file of any type
VERSION_PATT = r"({key}\s*[=:]\s*([\"\']?)){value}(\3\s*)".format(
    key=r'(["\']?)version\2',
    value=r"(?P<version>\d+\.\d+\.\d+[a-z0-9\-\+\.]*)",
)

# The valid types of increments you could make to a semantic version, and the
# functions they map to
INCREMENT_TYPES = {
    "major": semver.bump_major,
    "minor": semver.bump_minor,
    "patch": semver.bump_patch,
    "prerelease": semver.bump_prerelease,
}


def get_auto_detectable_file_names():
    # Get the name of the project directory
    project_name = os.path.basename(os.getcwd())
    return (
        # If a Node project
        "package.json",
        # If Node project contains a lockfile
        "package-lock.json",
        # If a WordPress theme
        "style.css",
        # If a WordPress plugin
        "{}.php".format(project_name),
        # If a Python project
        "setup.py",
        "setup.cfg",
        "pyproject.toml",
    )


# Increment the major, minor, or patch part of the given version string and
# return the incremented version
def bump_version(version, version_specifier):
    if version_specifier in INCREMENT_TYPES.keys():
        return INCREMENT_TYPES[version_specifier](version)
    else:
        # If we are not incrementing the version using one of the above
        # commands, we can assume the version specified is the explicit new
        # version to use
        return version_specifier


# The callback function for the substitution call that locates and increments
# the version in-place
def replace_version(version_specifier, version_match):
    return "".join((version_match[1], version_specifier, version_match[5]))


# Replace the version in the given file contents with the version matching the
# given version specifier
def replace_version_in_file_contents(file_contents, version_specifier):
    version_matches = re.search(VERSION_PATT, file_contents)
    if not version_matches:
        return (None, None, None)
    current_version = version_matches.group("version")
    new_version = bump_version(current_version, version_specifier)
    new_file_contents = re.sub(
        VERSION_PATT,
        partial(replace_version, new_version),
        file_contents,
        flags=re.IGNORECASE,
        count=1,
    )
    return (current_version, new_version, new_file_contents)


# Locate the version number in the specified file and increment it
def bump_version_for_file(version_specifier, file_path):
    try:
        with open(file_path, "r+") as file:
            file_contents = file.read()
            (
                current_version,
                new_version,
                new_file_contents,
            ) = replace_version_in_file_contents(file_contents, version_specifier)
            if not current_version:
                return (False, None, None)
            if new_file_contents == file_contents:
                return (False, None, None)
            file.truncate(0)
            file.seek(0)
            file.write(new_file_contents)
            print("{}: {} -> {}".format(file_path, current_version, new_version))
            return (True, current_version, new_version)
    except FileNotFoundError:
        print("{}: file not found".format(file_path), file=sys.stderr)
        return (False, None, None)


# Find files in the current project (according to the project type) that
# include version information
def get_default_file_paths():
    return [
        file_name
        for file_name in get_auto_detectable_file_names()
        if os.path.exists(file_name)
    ]


def abort_if_version_mismatch(file_results):
    if len(set(result.new_version for result in file_results)) > 1:
        print(
            "Aborting commit because not all bumped versions are equal", file=sys.stderr
        )
        sys.exit()


def bump_version_for_files(file_paths, version_specifier):
    file_results = []
    for file_path in file_paths:
        did_version_change, current_version, new_version = bump_version_for_file(
            file_path=file_path, version_specifier=version_specifier
        )
        if did_version_change:
            file_results.append(
                FileResult(
                    file_path=file_path,
                    current_version=current_version,
                    new_version=new_version,
                )
            )
    return file_results


def handle_git_operations(
    file_results, commit_message, tag_name=None, should_tag=False
):
    if not git.is_in_git_repository():
        return
    abort_if_version_mismatch(file_results)
    changed_result_paths = [result.file_path for result in file_results]
    git.add(changed_result_paths)
    print(f"Staging {', '.join(changed_result_paths)}")
    git.commit(commit_message)
    if not should_tag:
        return
    did_tag = git.tag(tag_name)
    if not did_tag:
        return
    print(f"Tagging commit as {tag_name}")


def version_specifier(arg_value):
    # Strip out 'v' prefix if included in the version specifier
    arg_value = re.sub(r"^v", "", arg_value)
    if arg_value in INCREMENT_TYPES.keys() or semver.Version.is_valid(arg_value):
        return arg_value
    else:
        raise argparse.ArgumentTypeError(
            "invalid version specifier (must be a valid semantic version)"
        )


def parse_cli_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("version_specifier", type=version_specifier)
    parser.add_argument(
        "file_paths",
        metavar="file",
        nargs="*",
        type=os.path.expanduser,
        default=get_default_file_paths(),
    )
    parser.add_argument("--no-commit", "-n", action="store_true")
    parser.add_argument("--no-tag", action="store_true")
    parser.add_argument(
        "--commit-message", "-m", default="Prepare v{new_version} release"
    )
    parser.add_argument("--tag-name", "-t", default="v{new_version}")
    return parser.parse_args()


def main():
    args = parse_cli_args()
    file_results = bump_version_for_files(args.file_paths, args.version_specifier)
    if not file_results:
        print("No files updated")
        return
    if not args.no_commit:
        new_version = file_results[0].new_version
        handle_git_operations(
            file_results=file_results,
            commit_message=args.commit_message.format(new_version=new_version),
            tag_name=args.tag_name.format(new_version=new_version),
            should_tag=not args.no_tag,
        )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from nose2.tools import params

import bumpanything.__main__ as bump
from tests import (
    BumpAnythingTestCase,
    create_mock_file,
    init_git_repo,
    read_mock_file,
    redirect_stderr,
    redirect_stdout,
    run_git_command,
    use_cli_args,
)


class TestGit(BumpAnythingTestCase):

    @redirect_stdout
    @redirect_stderr
    def test_version_mismatch(self, err, out):
        """should abort Git operations due to mismatch between multiple files"""
        file_name_1 = "package.json"
        file_name_2 = "package-lock.json"
        old_version_1 = "1.2.3"
        old_version_2 = "4.5.6"
        increment = "minor"
        new_version_1 = "1.3.0"
        new_version_2 = "4.6.0"
        file_contents_1 = f"""{{
            "name": "foo",
            "version": {old_version_1}
        }}"""
        file_contents_2 = f"""{{
            "lockfileVersion": 1,
            "name": "foo",
            "version": {old_version_2}
        }}"""
        with use_cli_args(increment):
            create_mock_file(file_name_1, file_contents_1)
            create_mock_file(file_name_2, file_contents_2)
            init_git_repo()
            with self.assertRaises(SystemExit):
                bump.main()
            self.assertIn(f'"version": {new_version_1}\n', read_mock_file(file_name_1))
            self.assertIn(f'"version": {new_version_2}\n', read_mock_file(file_name_2))
            self.assertIn("abort", err.getvalue().lower())

    @redirect_stdout
    def test_auto_commit_auto_tag(self, out):
        """should auto-commit and auto-tag release with Git after bumping version"""
        file_name = "package.json"
        old_version = "1.2.3"
        new_version = "1.3.0"
        file_contents = f"""{{
            "name": "foo",
            "version": {old_version}
        }}"""
        with use_cli_args(new_version):
            create_mock_file(file_name, file_contents)
            create_mock_file(file_name, file_contents)
            init_git_repo()
            bump.main()
            self.assertEqual(
                f"Prepare v{new_version} release",
                run_git_command("show", "-s", "--format=%B").strip(),
            )
            self.assertEqual(
                f"v{new_version}",
                run_git_command("describe", "--tags", "--exact-match").strip(),
            )

    @params(("--commit-message", "--tag-name"), ("-m", "-t"))
    @redirect_stdout
    def test_custom_commit_message_custom_tag_name(
        self, out, commit_msg_flag, tag_name_flag
    ):
        """should use custom commit message and custom tag name"""
        file_name = "package.json"
        old_version = "1.2.3"
        new_version = "1.3.0"
        commit_msg = "Release version {new_version}"
        tag_name = "release-{new_version}"
        file_contents = f"""{{
            "name": "foo",
            "version": {old_version}
        }}"""
        with use_cli_args(
            new_version, commit_msg_flag, commit_msg, tag_name_flag, tag_name
        ):
            create_mock_file(file_name, file_contents)
            create_mock_file(file_name, file_contents)
            init_git_repo()
            bump.main()
            self.assertEqual(
                commit_msg.format(new_version=new_version),
                run_git_command("show", "-s", "--format=%B").strip(),
            )
            self.assertEqual(
                tag_name.format(new_version=new_version),
                run_git_command("describe", "--tags", "--exact-match").strip(),
            )

    @redirect_stdout
    def test_auto_commit_no_tag(self, out):
        """should auto-commit but not tag release"""
        file_name = "package.json"
        old_version = "1.2.3"
        new_version = "1.3.0"
        file_contents = f"""{{
            "name": "foo",
            "version": {old_version}
        }}"""
        with use_cli_args(new_version, "--no-tag"):
            create_mock_file(file_name, file_contents)
            create_mock_file(file_name, file_contents)
            init_git_repo()
            bump.main()
            self.assertEqual(
                f"Prepare v{new_version} release",
                run_git_command("show", "-s", "--format=%B").strip(),
            )
            self.assertIn(
                "fatal",
                run_git_command("describe", "--tags", "--exact-match").strip(),
            )

    @redirect_stdout
    def test_auto_commit_existing_tag(self, out):
        """should auto-commit but not tag release due to existing tag"""
        file_name = "package.json"
        old_version = "0.8.0"
        new_version = "1.0.0"
        file_contents = f"""{{
            "name": "foo",
            "version": {old_version}
        }}"""
        with use_cli_args(new_version):
            create_mock_file(file_name, file_contents)
            create_mock_file(file_name, file_contents)
            init_git_repo()
            # Tag the initial commit created in init_git_repo()
            run_git_command("tag", f"v{new_version}")
            bump.main()
            self.assertEqual(
                f"Prepare v{new_version} release",
                run_git_command("show", "-s", "--format=%B").strip(),
            )
            self.assertIn(
                "fatal",
                run_git_command("describe", "--tags", "--exact-match").strip(),
            )

    @params("--no-commit", "-n")
    @redirect_stdout
    def test_no_commit(self, out, no_commit_flag):
        """should not commit when --no-commit is supplied"""
        file_name = "package.json"
        old_version = "0.8.0"
        new_version = "1.0.0"
        file_contents = f"""{{
            "name": "foo",
            "version": {old_version}
        }}"""
        with use_cli_args(new_version, no_commit_flag):
            create_mock_file(file_name, file_contents)
            create_mock_file(file_name, file_contents)
            init_git_repo()
            bump.main()
            self.assertEqual(
                "Initial commit",
                run_git_command("show", "-s", "--format=%B").strip(),
            )
            self.assertIn(
                "fatal",
                run_git_command("describe", "--tags", "--exact-match").strip(),
            )
